@@ -7,11 +7,11 @@ import time
 import atexit
 import random
 import sys
+import logging
 
 from zombie.proxy.client import ZombieProxyClient
 
 __all__ = ['ZombieProxyServer']
-__proxy_instances__ = []
 
 
 class PipeWorker(threading.Thread):
@@ -23,34 +23,54 @@ class PipeWorker(threading.Thread):
     def __init__(self, pipe):
         super(PipeWorker, self).__init__()
         self.pipe = pipe
-        self.setDaemon(True)
+        self.daemon = True
+        self.log = logging.getLogger(__name__)
 
-    def __worker__(self, pipe):
+    def __worker(self, pipe):
         while True:
             line = pipe.readline()
             if line:
-                sys.stdout.write(line)
+                self.log.debug(line[:-1])
             else:
                 break
 
     def run(self):
         try:
-            self.__worker__(self.pipe)
+            self.__worker(self.pipe)
         except Exception as e:
             try:
-                sys.stdout.write(e)
-            except:  # pragma: nocover
+                self.log.error(e)
+            except:
                 pass
 
+__server_instance__ = None
+proxy_path = os.path.join(
+    os.path.dirname(os.path.abspath(__file__)),
+    'server.js'
+)
 
+
+def singleton(cls):
+    instances = {}
+
+    def ZombieProxyServer(*args, **kwargs):
+        if cls not in instances:
+            instances[cls] = cls(*args, **kwargs)
+            global __server_instance__
+            __server_instance__ = instances[cls]
+        return instances[cls]
+    return ZombieProxyServer
+
+
+@singleton
 class ZombieProxyServer(object):
 
     def __init__(self, socket=None, wait=True):
         """
         Spawns a node.js subprocess that listens on a TCP socket.
-        A ZombieProxyClient streams data to the server, which
-        evaluates it as Javascript, passes it on to a Zombie.js
-        Browser object, and returns the results.
+        A :class:`zombie.proxy.client.ZombieProxyClient` streams data to
+        the server, which evaluates it as Javascript, passes it on to
+        a zombie.js Browser object, and returns the results.
 
         :param socket: a (random, by default) filepath representing the
                        intended TCP socket location
@@ -60,13 +80,17 @@ class ZombieProxyServer(object):
         socket = socket or '/tmp/zombie-%s.sock' % random.randint(0, 10000)
 
         self.socket = socket
+
+        # Kill the node process when finished
+        atexit.register(__kill_node_processes__)
+
         #
         # Spawn the node proxy server in a subprocess.
         # This is a simple socket server that listens for data,
         # evaluates it as Javascript, and passes the eval'ed
         # input to a Zombie.js Browser object.
         #
-        args = ['env', 'node', self.__proxy_path__(), self.socket]
+        args = ['env', 'node', proxy_path, self.socket]
         self.child = subprocess.Popen(
             args,
             stdin=subprocess.PIPE,
@@ -74,7 +98,7 @@ class ZombieProxyServer(object):
             stderr=subprocess.STDOUT
         )
         self.child.stdin.close()
-
+        PipeWorker(self.child.stdout).start()
         if wait:
             # Wait until we can ping the node.js server
             client = ZombieProxyClient(socket)
@@ -93,34 +117,16 @@ class ZombieProxyServer(object):
                     break
                 time.sleep(.1)
 
-        global __proxy_instances__
-        __proxy_instances__.append((self.child, self.socket))
-
-        #
-        # Start a thread to monitor and redirect the
-        # subprocess stdout and stderr to the console.
-        #
-        PipeWorker(self.child.stdout).start()
-
-    @classmethod
-    def __proxy_path__(self):
-        return os.path.join(
-            os.path.dirname(os.path.abspath(__file__)),
-            'server.js'
-        )
-
 
 # When this process ends, ensure all node subprocesses terminate
 def __kill_node_processes__():  # pragma: nocover
-    for child, socket in __proxy_instances__:
+    instance = __server_instance__
+    if instance:
         from os import path
-        if hasattr(child, 'kill'):
-            child.kill()
+        if hasattr(instance.child, 'kill'):
+            instance.child.kill()
 
         # Cleanup the closed socket
-        if path.exists(socket):
+        if path.exists(instance.socket):
             from os import remove
-            remove(socket)
-atexit.register(__kill_node_processes__)
-signal.signal(signal.SIGTERM, lambda signum, stack_frame: exit(1))
-signal.signal(signal.SIGINT, lambda signum, stack_frame: exit(1))
+            remove(instance.socket)
